@@ -4,26 +4,17 @@ from __future__ import annotations
 Low level operations on packfiles.
 """
 
-import mmap
+import os
 import zlib
-
-
-# dict[str path, mmap.mmap]
-MAPPED_CACHE = {}
-
-
-def _get_buffer(path) -> mmap.mmap:
-    if path in MAPPED_CACHE and not MAPPED_CACHE[path].closed:
-        return MAPPED_CACHE[path]
-
-    with open(path, 'rb') as raw:
-        m = mmap.mmap(raw.fileno(), 0, access=mmap.ACCESS_READ)
-
-    MAPPED_CACHE[path] = m
-    return m
+from io import BytesIO
 
 
 class Packs:
+    # {'path': (mtime, buffer)}
+    bytes_io_cache: dict[str, tuple[float, BytesIO]] = {}
+    # {'path': (mtime, {'hash': offset})}
+    packs_index_cache: dict[str, tuple[float, dict[str, int]]] = {}
+
     def search_idx(
             self,
             idx_path: str,
@@ -44,54 +35,47 @@ class Packs:
             the index file, returns None.
         """
 
-        layer1_idx = int(hash_[:2], 16) - 1
-        found = False
+        mtime = os.stat(idx_path, follow_symlinks=False).st_mtime
+        cache = self.packs_index_cache.get(idx_path)
+        if cache and cache[0] == mtime:
+            if not rt_offset:
+                return hash_ in cache[1]
+            return cache[1].get(hash_)
 
-        file = _get_buffer(idx_path)
+        hashes = []
+        offsets = []
 
-        if layer1_idx > 0:
-            file.seek(8 + 4 * layer1_idx)
-            files_before = int.from_bytes(file.read(4), 'big')
-        else:
-            files_before = 0
+        with open(idx_path, 'rb') as file:
 
-        file.seek(1028)
-        total_files = int.from_bytes(file.read(4), 'big')
+            file.seek(1028)
+            total_files = int.from_bytes(file.read(4), 'big')
 
-        file.seek(
-            1032  # end of fanout_layer1
-            + 20 * files_before  # each will have 20 bytes
-        )
+            file.seek(
+                1032  # end of fanout_layer1
+                # + 20 * files_before  # each will have 20 bytes
+            )
 
-        while not found:
-            file_hash_bytes = file.read(20)
-            file_hash = hex(
-                int.from_bytes(file_hash_bytes, 'big')
-            )[2:].zfill(40)
+            for _ in range(total_files):
+                file_hash_bytes = file.read(20)
+                file_hash = hex(
+                    int.from_bytes(file_hash_bytes, 'big')
+                )[2:].zfill(40)
 
-            if hash_ == file_hash:
-                found = True
-                break
+                hashes.append(file_hash)
 
-            elif int(file_hash[:2], 16) > layer1_idx + 1:
-                break
-
-            elif file.tell() >= 1032 + 20 * total_files:
-                break
-
-            files_before += 1
-
-        if not rt_offset:
-            return found
-
-        if found:
             file.seek(
                 1032
                 + 20 * total_files  # jump layer2
                 + 4 * total_files  # jump layer3
-                + 4 * files_before
             )
-            return int.from_bytes(file.read(4), 'big')
+
+            for _ in range(total_files):
+                offsets.append(int.from_bytes(file.read(4), 'big'))
+
+            new_cache = dict(zip(hashes, offsets))
+            self.packs_index_cache[idx_path] = mtime, new_cache
+
+            return self.search_idx(idx_path, hash_, rt_offset)
 
     def get_content_by_offset(
             self, pack_path: str, offset: int
@@ -104,7 +88,7 @@ class Packs:
                               None it is a delta object
         """
 
-        file = _get_buffer(pack_path)
+        file = self._get_buffer(pack_path)
         file.seek(offset)
 
         int_ = int.from_bytes(file.read(1), 'big')
@@ -114,7 +98,7 @@ class Packs:
         bit_shift = 4
 
         if type_ == '110':  # delta type
-            return
+            return None
 
         msb = binary.startswith('1')
         # size = binary[4:]
@@ -126,6 +110,7 @@ class Packs:
             bit_shift += 7
             msb = binary.startswith('1')
 
+        # why + 11? I don't know, it works
         return zlib.decompress(file.read(obj_size + 11))
 
     def get_idx_of_pack(self, pack: str) -> str:
@@ -133,3 +118,17 @@ class Packs:
         return pack.removesuffix('pack') + 'idx'
         """
         return pack.removesuffix('pack') + 'idx'
+
+    def _get_buffer(self, path) -> BytesIO:
+        mtime = os.stat(path, follow_symlinks=False).st_mtime
+        cache = self.bytes_io_cache.get(path)
+        if cache and cache[0] == mtime:
+            c = cache[1]
+            c.seek(0)
+            return c
+
+        with open(path, 'rb') as raw:
+            c = BytesIO(raw.read())
+
+        self.bytes_io_cache[path] = mtime, c
+        return c
